@@ -7,10 +7,13 @@
 
 #include <string.h>
 
+#include <isc/boolean.h>
 #include <isc/mem.h>
 #include <isc/buffer.h>
+#include <isc/result.h>
 #include <dns/types.h>
 #include <dns/name.h>
+#include <dns/result.h>
 
 #include "log.h"
 
@@ -103,11 +106,15 @@ extern isc_boolean_t verbose_checks; /* from settings.c */
 /* If no argument index list is given to the nonnull attribute,
  * all pointer arguments are marked as non-null. */
 #define ATTR_NONNULLS     ATTR_NONNULL()
-#ifdef __GNUC__
+#if defined(__COVERITY__) || defined(__clang_analyzer__)
 #define ATTR_NONNULL(...) __attribute__((nonnull(__VA_ARGS__)))
-#define ATTR_CHECKRESULT __attribute__((warn_unused_result))
 #else
 #define ATTR_NONNULL(...)
+#endif
+
+#if defined(__GNUC__)
+#define ATTR_CHECKRESULT __attribute__((warn_unused_result))
+#else
 #define ATTR_CHECKRESULT
 #endif
 
